@@ -16,7 +16,7 @@ Classes for drawing graphics items on QGraphicsScene.
 :date:    4.4.2018
 """
 
-from math import atan2, sin, cos, pi
+from math import sin, cos, pi, radians
 from PySide2.QtCore import Qt, Slot, QPointF, QLineF, QRectF, QVariantAnimation
 from PySide2.QtWidgets import (
     QGraphicsItem,
@@ -26,12 +26,14 @@ from PySide2.QtWidgets import (
     QStyle,
     QToolTip,
 )
-from PySide2.QtGui import QColor, QPen, QBrush, QPainterPath, QLinearGradient, QFont
+from PySide2.QtGui import QColor, QPen, QBrush, QPainterPath, QLinearGradient, QFont, QCursor, QPainterPathStroker
 from PySide2.QtSvg import QGraphicsSvgItem, QSvgRenderer
-from spinetoolbox.mvcmodels.resource_filter_model import ResourceFilterModel
-from spinetoolbox.helpers import busy_effect
+from spinetoolbox.helpers import color_from_index
 from .project_item_icon import ConnectorButton
 
+LINK_COLOR = color_from_index(0, 2, base_hue=60)
+JUMP_COLOR = color_from_index(1, 2, base_hue=60)
+
 
 class LinkBase(QGraphicsPathItem):
     """Base class for Link and LinkDrawer.
@@ -39,6 +41,8 @@ class LinkBase(QGraphicsPathItem):
     Mainly provides the ``update_geometry`` method for 'drawing' the link on the scene.
     """
 
+    _COLOR = QColor(0, 0, 0, 0)
+
     def __init__(self, toolbox, src_connector, dst_connector):
         """
         Args:
@@ -52,6 +56,27 @@ class LinkBase(QGraphicsPathItem):
         self.dst_connector = dst_connector
         self.arrow_angle = pi / 4
         self.setCursor(Qt.PointingHandCursor)
+        self._guide_path = None
+        self._pen = QPen(self._COLOR)
+        self._pen.setWidthF(self.magic_number)
+        self._pen.setJoinStyle(Qt.MiterJoin)
+        self.setPen(self._pen)
+        self.selected_pen = QPen(self.outline_color, 2, Qt.DotLine)
+        self.normal_pen = QPen(self.outline_color, 1)
+        self._outline = QGraphicsPathItem(self)
+        self._outline.setFlag(QGraphicsPathItem.ItemStacksBehindParent)
+        self._outline.setPen(self.normal_pen)
+        self._stroker = QPainterPathStroker()
+        self._stroker.setWidth(self.magic_number)
+        self._stroker.setJoinStyle(Qt.MiterJoin)
+        self._shape = QPainterPath()
+
+    def shape(self):
+        return self._shape
+
+    @property
+    def outline_color(self):
+        return self._COLOR.darker()
 
     @property
     def magic_number(self):
@@ -86,45 +111,97 @@ class LinkBase(QGraphicsPathItem):
         if curved_links is None:
             qsettings = self._toolbox.qsettings()
             curved_links = qsettings.value("appSettings/curvedLinks", defaultValue="false") == "true"
-        guide_path = self._make_guide_path(curved_links)
-        self.do_update_geometry(guide_path)
+        self._guide_path = self._make_guide_path(curved_links)
+        self._do_update_geometry()
 
-    def do_update_geometry(self, guide_path):
-        """Sets the path for this item.
+    def guide_path(self):
+        """For tests."""
+        return self._guide_path
+
+    def _do_update_geometry(self):
+        """Sets the path for this item."""
+        path = QPainterPath(self._guide_path)
+        self._add_arrow_path(path)
+        self._add_ellipse_path(path)
+        self.setPath(path)
+        stroke = self._stroker.createStroke(path)
+        self._outline.setPath(stroke)
+        self._shape.clear()
+        self._shape.addPath(stroke)
+
+    def _add_ellipse_path(self, path):
+        """Adds an ellipse for the link's base.
 
         Args:
-            guide_path (QPainterPath)
-        """
-        ellipse_path = self._make_ellipse_path()
-        connecting_path = self._make_connecting_path(guide_path)
-        arrow_path = self._make_arrow_path(guide_path)
-        path = ellipse_path + connecting_path + arrow_path
-        self.setPath(path.simplified())
-
-    def _make_ellipse_path(self):
-        """Returns an ellipse path for the link's base.
-
-        Returns:
             QPainterPath
         """
-        ellipse_path = QPainterPath()
-        rect = QRectF(0, 0, 1.6 * self.magic_number, 1.6 * self.magic_number)
+        radius = 0.5 * self.magic_number
+        rect = QRectF(0, 0, radius, radius)
         rect.moveCenter(self.src_center)
-        ellipse_path.addEllipse(rect)
-        return ellipse_path
+        path.addEllipse(rect)
+
+    def _get_joint_angle(self):
+        return radians(self._guide_path.angleAtPercent(0.99))
+
+    def _add_arrow_path(self, path):
+        """Returns an arrow path for the link's tip.
+
+        Args:
+            QPainterPath
+        """
+        angle = self._get_joint_angle()
+        arrow_p0 = self.dst_center + 0.5 * self.magic_number * self._get_dst_offset()
+        d1 = QPointF(sin(angle + self.arrow_angle), cos(angle + self.arrow_angle))
+        d2 = QPointF(sin(angle + (pi - self.arrow_angle)), cos(angle + (pi - self.arrow_angle)))
+        arrow_diag = 1.5 / sin(self.arrow_angle)
+        arrow_p1 = arrow_p0 - d1 * arrow_diag
+        arrow_p2 = arrow_p0 - d2 * arrow_diag
+        path.moveTo(arrow_p1)
+        path.lineTo(arrow_p0)
+        path.lineTo(arrow_p2)
+        path.closeSubpath()
+
+    @staticmethod
+    def _get_offset(button):
+        return {"top": QPointF(0, -1), "left": QPointF(-1, 0), "bottom": QPointF(0, 1), "right": QPointF(1, 0)}[
+            button.position
+        ]
 
     def _get_src_offset(self):
-        return {"left": QPointF(-1, 0), "bottom": QPointF(0, 1), "right": QPointF(1, 0)}[self.src_connector.position]
+        return self._get_offset(self.src_connector)
 
-    def _get_dst_offset(self, c1):
-        if not self.dst_connector:
-            guide_path = QPainterPath(self.src_center)
-            guide_path.quadTo(c1, self.dst_center)
-            line = self._get_joint_line(guide_path).unitVector()
-            return QPointF(-line.dx(), -line.dy())
-        return {"left": QPointF(-1, 0), "bottom": QPointF(0, 1), "right": QPointF(1, 0)}[self.dst_connector.position]
+    def _get_dst_offset(self):
+        return self._get_offset(self.dst_connector)
 
-    def _make_guide_path(self, curved_links):
+    def _find_new_point(self, points, target):
+        """Finds a new point that approximates points to target in a smooth trajectory.
+        Returns the new point, or None if no need for approximation.
+
+        Args:
+            points (list(QPointF))
+            target (QPointF)
+
+        Returns:
+            QPointF or None
+        """
+        line = QLineF(*points[-2:])
+        line_to_target = QLineF(points[-1], target)
+        angle = line.angleTo(line_to_target)
+        corrected_angle = angle if angle < 180 else angle - 360
+        if abs(corrected_angle) <= 90:
+            return None
+        sign = abs(corrected_angle) // corrected_angle
+        new_angle = line.angle() + 90 * sign
+        foot = sin if angle > 0 else cos
+        new_length = max(abs(foot(radians(angle))) * line_to_target.length(), 3 * self.magic_number)
+        line_to_target.setAngle(new_angle)
+        line_to_target.setLength(new_length)
+        return line_to_target.center()
+
+    def _close_enough(self, p1, p2):
+        return (p1 - p2).manhattanLength() < 2 * self.magic_number
+
+    def _make_guide_path(self, curved_links=False):
         """Returns a 'narrow' path connecting this item's source and destination.
 
         Args:
@@ -133,202 +210,70 @@ class LinkBase(QGraphicsPathItem):
         Returns:
             QPainterPath
         """
-        path = QPainterPath(self.src_center)
-        if not curved_links:
-            path.lineTo(self.dst_center)
+        c_factor = 3 * self.magic_number
+        src = self.src_center + c_factor * self._get_src_offset()
+        dst = self.dst_center + c_factor * self._get_dst_offset()
+        src_points = [self.src_center, src]
+        dst_points = [self.dst_center, dst]
+        while True:
+            # Bring source points closer to destination
+            new_src = self._find_new_point(src_points, dst)
+            if new_src is not None:
+                src_points.append(new_src)
+                src = new_src
+            if self._close_enough(src, dst):
+                break
+            # Bring destination points closer to source
+            new_dst = self._find_new_point(dst_points, src)
+            if new_dst is not None:
+                dst_points.append(new_dst)
+                dst = new_dst
+            if self._close_enough(src, dst):
+                break
+            if new_src is new_dst is None:
+                break
+        points = src_points + list(reversed(dst_points))
+        points = list(map(lambda xy: QPointF(*xy), dict.fromkeys((p.x(), p.y()) for p in points)))
+        if len(points) == 1:
+            path = QPainterPath(points[0])
+            path.lineTo(points[0] + QPointF(1, 1))
             return path
-        c_min = 2 * self.magic_number
-        c_max = 8 * self.magic_number
-        c_factor = QLineF(self.src_center, self.dst_center).length() / 2
-        c_factor = min(c_factor, c_max)
-        c_factor = max(c_factor, c_min)
-        c1 = self.src_center + c_factor * self._get_src_offset()
-        c2 = self.dst_center + c_factor * self._get_dst_offset(c1)
-        path.cubicTo(c1, c2, self.dst_center)
+        # Correct last point so it doesn't go beyond the arrow
+        head = QPainterPath(points[-2])
+        head.lineTo(points[-1])
+        points[-1] = head.pointAtPercent(1 - head.percentAtLength(self.magic_number))
+        # Make path
+        path = QPainterPath(points.pop(0))
+        if not curved_links:
+            for p1 in points:
+                path.lineTo(p1)
+            return path
+        for p1, p2 in zip(points[:-2], points[1:-1]):
+            path.quadTo(p1, (p1 + p2) / 2)
+        path.quadTo(points[-2], points[-1])
         return path
 
-    def _points_and_angles_from_path(self, path):
-        """Returns a list of representative points and angles from given path.
+    def itemChange(self, change, value):
+        """Wipes out the link when removed from scene."""
+        if change == QGraphicsItem.GraphicsItemChange.ItemSceneHasChanged and value is None:
+            self.wipe_out()
+        return super().itemChange(change, value)
 
-        Args:
-            path (QPainterPath)
-
-        Returns:
-            list(QPointF): points
-            list(float): angles
-        """
-        count = min(int(100 * (1.0 - path.percentAtLength(self.src_rect.width() / 2))) + 2, 100)
-        percents = [k / 100 for k in range(count)]
-        points = list(map(path.pointAtPercent, percents))
-        angles = list(map(path.angleAtPercent, percents))
-        return points, angles
-
-    def _make_connecting_path(self, guide_path):
-        """Returns a 'thick' path connecting source and destination, by following the given 'guide' path.
-
-        Args:
-            guide_path (QPainterPath)
-
-        Returns:
-            QPainterPath
-        """
-        points, angles = self._points_and_angles_from_path(guide_path)
-        outgoing_points = []
-        incoming_points = []
-        for point, angle in zip(points, angles):
-            off = self._radius_from_point_and_angle(point, angle)
-            outgoing_points.append(point + off)
-            incoming_points.insert(0, point - off)
-        p0 = guide_path.pointAtPercent(0)
-        a0 = guide_path.angleAtPercent(0)
-        off0 = self._radius_from_point_and_angle(p0, a0)
-        curve_path = QPainterPath(p0 + off0)
-        self._follow_points(curve_path, outgoing_points)
-        curve_path.lineTo(incoming_points[0])
-        self._follow_points(curve_path, incoming_points)
-        curve_path.lineTo(p0 - off0)
-        curve_path.closeSubpath()
-        curve_path.setFillRule(Qt.WindingFill)
-        return curve_path.simplified()
-
-    @staticmethod
-    def _follow_points(curve_path, points):
-        points = iter(points)
-        for p0 in points:
-            p1 = next(points, None)
-            if p1 is None:
-                break
-            curve_path.quadTo(p0, p1)
-
-    def _radius_from_point_and_angle(self, point, angle):
-        line = QLineF()
-        line.setP1(point)
-        line.setAngle(angle)
-        normal = line.normalVector()
-        normal.setLength(self.magic_number / 2)
-        return QPointF(normal.dx(), normal.dy())
-
-    def _make_arrow_path(self, guide_path):
-        """Returns an arrow path for the link's tip.
-
-        Args:
-            guide_path (QPainterPath): A narrow path connecting source and destination,
-                used to determine the arrow orientation.
-
-        Returns:
-            QPainterPath
-        """
-        angle = self._get_joint_angle(guide_path)
-        arrow_p0 = self.dst_center
-        d1 = QPointF(sin(angle + self.arrow_angle), cos(angle + self.arrow_angle))
-        d2 = QPointF(sin(angle + (pi - self.arrow_angle)), cos(angle + (pi - self.arrow_angle)))
-        arrow_diag = self.magic_number / sin(self.arrow_angle)
-        arrow_p1 = arrow_p0 - d1 * arrow_diag
-        arrow_p2 = arrow_p0 - d2 * arrow_diag
-        arrow_path = QPainterPath(arrow_p1)
-        arrow_path.lineTo(arrow_p0)
-        arrow_path.lineTo(arrow_p2)
-        arrow_path.closeSubpath()
-        return arrow_path
-
-    def _get_joint_line(self, guide_path):
-        t = 1.0 - guide_path.percentAtLength(self.src_rect.width() / 2)
-        t = max(t, 0.01)
-        src = guide_path.pointAtPercent(t - 0.01)
-        dst = guide_path.pointAtPercent(t)
-        return QLineF(src, dst)
-
-    def _get_joint_angle(self, guide_path):
-        line = self._get_joint_line(guide_path)
-        return atan2(-line.dy(), line.dx())
+    def wipe_out(self):
+        """Removes any trace of this item from the system."""
 
 
-class _LinkIcon(QGraphicsEllipseItem):
-    """An icon to show over a Link."""
+class _IconBase(QGraphicsEllipseItem):
+    """Base class for icons to show over a Link."""
 
-    def __init__(self, x, y, w, h, parent):
+    def __init__(self, x, y, w, h, parent, tooltip=None):
         super().__init__(x, y, w, h, parent)
-        self._parent = parent
-        color = QColor("slateblue")
-        self.setBrush(qApp.palette().window())  # pylint: disable=undefined-variable
-        self._text_item = QGraphicsTextItem(self)
-        font = QFont('Font Awesome 5 Free Solid')
-        self._text_item.setFont(font)
-        self._text_item.setDefaultTextColor(color)
-        self._svg_item = QGraphicsSvgItem(self)
-        self._datapkg_renderer = QSvgRenderer()
-        self._datapkg_renderer.load(":/icons/datapkg.svg")
-        self.setFlag(QGraphicsItem.ItemIsSelectable, enabled=False)
-
-    def update_icon(self):
-        """Sets the icon (filter, datapkg, or none), depending on Connection state."""
-        connection = self._parent.connection
-        if connection.use_datapackage:
-            self.setVisible(True)
-            self._svg_item.setVisible(True)
-            self._svg_item.setSharedRenderer(self._datapkg_renderer)
-            scale = 0.8 * self.rect().width() / self._datapkg_renderer.defaultSize().width()
-            self._svg_item.setScale(scale)
-            self._svg_item.setPos(0, 0)
-            self._svg_item.setPos(self.sceneBoundingRect().center() - self._svg_item.sceneBoundingRect().center())
-            self._text_item.setVisible(False)
-            return
-        if connection.has_filters():
-            self.setVisible(True)
-            self._text_item.setVisible(True)
-            self._text_item.setPlainText("\uf0b0")
-            self._text_item.setPos(self.sceneBoundingRect().center() - self._text_item.sceneBoundingRect().center())
-            self._svg_item.setVisible(False)
-            return
-        self.setVisible(False)
-        self._text_item.setVisible(False)
-        self._svg_item.setVisible(False)
-
-    def wipe_out(self):
-        """Cleans up icon's resources."""
-        self._text_item.deleteLater()
-        self._svg_item.deleteLater()
-        self._datapkg_renderer.deleteLater()
-
-
-class _JumpIcon(QGraphicsEllipseItem):
-    """An icon to show over a JumpLink."""
-
-    _NORMAL_COLOR = QColor("black")
-    _ISSUE_COLOR = QColor("red")
-
-    def __init__(self, x, y, w, h, jump_link):
-        super().__init__(x, y, w, h, jump_link)
-        self._jump_link = jump_link
-        self.setBrush(qApp.palette().window())  # pylint: disable=undefined-variable
-        self._text_item = QGraphicsTextItem(self)
-        font = QFont('Font Awesome 5 Free Solid')
-        self._text_item.setFont(font)
-        self.setFlag(QGraphicsItem.ItemIsSelectable, enabled=False)
+        if tooltip:
+            self.setToolTip(tooltip)
         self.setAcceptHoverEvents(True)
-
-    def update_icon(self):
-        """Sets the icon depending on Jump state."""
-        issues = self._jump_link.issues()
-        if issues:
-            self._text_item.setDefaultTextColor(self._ISSUE_COLOR)
-            self._text_item.setPlainText("\uf06a")
-            self._text_item.setPos(
-                self.sceneBoundingRect().center() - self._text_item.sceneBoundingRect().center() + QPointF(0.0, 0.5)
-            )
-            self.setToolTip(issues[0])
-        else:
-            self._text_item.setDefaultTextColor(self._NORMAL_COLOR)
-            self._text_item.setPlainText("\uf1ce")
-            self._text_item.setPos(
-                self.sceneBoundingRect().center() - self._text_item.sceneBoundingRect().center() + QPointF(0.0, 0.5)
-            )
-            self.setToolTip("")
-
-    def wipe_out(self):
-        """Cleans up icon's resources."""
-        self._text_item.deleteLater()
-        self._renderer.deleteLater()
+        self.setFlag(QGraphicsItem.ItemIsSelectable, enabled=False)
+        self.setFlag(QGraphicsItem.ItemIsSelectable, enabled=False)
+        self.setBrush(qApp.palette().window())  # pylint: disable=undefined-variable
 
     def hoverEnterEvent(self, event):
         QToolTip.showText(event.screenPos(), self.toolTip())
@@ -337,10 +282,164 @@ class _JumpIcon(QGraphicsEllipseItem):
         QToolTip.hideText()
 
 
-class Link(LinkBase):
+class _SvgIcon(_IconBase):
+    """A svg icon to show over a Link."""
+
+    def __init__(self, x, y, w, h, parent, path, tooltip=None):
+        super().__init__(x, y, w, h, parent, tooltip=tooltip)
+        self._svg_item = QGraphicsSvgItem(self)
+        self._renderer = QSvgRenderer()
+        self._renderer.load(path)
+        self._svg_item.setSharedRenderer(self._renderer)
+        scale = 0.8 * self.rect().width() / self._renderer.defaultSize().width()
+        self._svg_item.setScale(scale)
+        self._svg_item.setPos(0, 0)
+        self._svg_item.setPos(self.sceneBoundingRect().center() - self._svg_item.sceneBoundingRect().center())
+
+    def wipe_out(self):
+        """Cleans up icon's resources."""
+        self._svg_item.deleteLater()
+        self._renderer.deleteLater()
+        self.scene().removeItem(self)
+
+
+class _TextIcon(_IconBase):
+    """A font awesome icon to show over a Link."""
+
+    def __init__(self, x, y, w, h, parent, char, tooltip=None, color=None):
+        super().__init__(x, y, w, h, parent, tooltip=tooltip)
+        if color is None:
+            color = QColor("slateblue")
+        self._text_item = QGraphicsTextItem(self)
+        font = QFont('Font Awesome 5 Free Solid')
+        self._text_item.setFont(font)
+        self._text_item.setDefaultTextColor(color)
+        self._text_item.setPlainText(char)
+        self._text_item.setPos(0, 0)
+        self._text_item.setPos(self.sceneBoundingRect().center() - self._text_item.sceneBoundingRect().center())
+
+    def wipe_out(self):
+        """Cleans up icon's resources."""
+        self._text_item.deleteLater()
+        self.scene().removeItem(self)
+
+
+class JumpOrLink(LinkBase):
+    """Base class for Jump and Link."""
+
+    def __init__(self, toolbox, src_connector, dst_connector):
+        super().__init__(toolbox, src_connector, dst_connector)
+        self.setFlag(QGraphicsItem.ItemIsSelectable, enabled=True)
+        self.setFlag(QGraphicsItem.ItemIsFocusable, enabled=True)
+        self._icon_extent = 4 * self.magic_number
+        self._icons = []
+        self._anim = self._make_execution_animation()
+        self.update_geometry()
+
+    @property
+    def item(self):
+        raise NotImplementedError()
+
+    def _do_update_geometry(self):
+        """See base class."""
+        super()._do_update_geometry()
+        self._place_icons()
+
+    def _place_icons(self):
+        delta = 0.6 / (len(self._icons) + 1)
+        percent = 0.2 + delta
+        for icon in self._icons:
+            center = self._guide_path.pointAtPercent(percent)
+            icon.setPos(center - 0.5 * QPointF(self._icon_extent, self._icon_extent))
+            percent += delta
+
+    def mousePressEvent(self, e):
+        """Ignores event if there's a connector button underneath,
+        to allow creation of new links.
+
+        Args:
+            e (QGraphicsSceneMouseEvent): Mouse event
+        """
+        if any(isinstance(x, ConnectorButton) for x in self.scene().items(e.scenePos())):
+            e.ignore()
+
+    def contextMenuEvent(self, e):
+        """Selects the link and shows context menu.
+
+        Args:
+            e (QGraphicsSceneMouseEvent): Mouse event
+        """
+        self.setSelected(True)
+        self._toolbox.show_link_context_menu(e.screenPos(), self)
+
+    def paint(self, painter, option, widget=None):
+        """Sets a dashed pen if selected."""
+        if option.state & QStyle.State_Selected:
+            option.state &= ~QStyle.State_Selected
+            self._outline.setPen(self.selected_pen)
+            for icon in self._icons:
+                icon.setPen(self.selected_pen)
+        else:
+            self._outline.setPen(self.normal_pen)
+            for icon in self._icons:
+                icon.setPen(self.normal_pen)
+        super().paint(painter, option, widget)
+
+    def shape(self):
+        shape = super().shape()
+        for icon in self._icons:
+            path = QPainterPath()
+            path.addEllipse(icon.sceneBoundingRect())
+            shape += path
+        return shape
+
+    def wipe_out(self):
+        """Removes any trace of this item from the system."""
+        self.src_connector.links.remove(self)
+        self.dst_connector.links.remove(self)
+
+    def _make_execution_animation(self):
+        """Returns an animation to play when execution 'passes' through this link.
+
+        Returns:
+            QVariantAnimation
+        """
+        animation = QVariantAnimation()
+        animation.setStartValue(0.0)
+        animation.setEndValue(1.0)
+        animation.valueChanged.connect(self._handle_execution_animation_value_changed)
+        animation.finished.connect(lambda: self.setPen(self._pen))
+        return animation
+
+    def run_execution_animation(self):
+        """Runs execution animation."""
+        qsettings = self._toolbox.qsettings()
+        duration = int(qsettings.value("appSettings/dataFlowAnimationDuration", defaultValue="100"))
+        self._anim.setDuration(duration)
+        self._anim.start()
+
+    @Slot(object)
+    def _handle_execution_animation_value_changed(self, step):
+        exec_color = QColor("red")
+        gradient = QLinearGradient(self.src_center, self.dst_center)
+        delta = 8 * self.magic_number / QLineF(self.src_center, self.dst_center).length()
+        gradient.setColorAt(0, self._COLOR)
+        gradient.setColorAt(max(0.0, step - delta), self._COLOR)
+        gradient.setColorAt(step, exec_color)
+        gradient.setColorAt(min(1.0, step + delta), self._COLOR)
+        gradient.setColorAt(1.0, self._COLOR)
+        pen = QPen(self._pen)
+        pen.setBrush(gradient)
+        self.setPen(pen)
+
+
+class Link(JumpOrLink):
     """A graphics item to represent the connection between two project items."""
 
-    _COLOR = QColor(255, 255, 0, 200)
+    _COLOR = LINK_COLOR
+    _MEMORY = "\uf538"
+    _FILTERS = "\uf0b0"
+    _DATAPACKAGE = ":/icons/datapkg.svg"
 
     def __init__(self, toolbox, src_connector, dst_connector, connection):
         """
@@ -352,35 +451,20 @@ class Link(LinkBase):
         """
         super().__init__(toolbox, src_connector, dst_connector)
         self._connection = connection
-        self.selected_pen = QPen(Qt.black, 1, Qt.DashLine)
-        self.normal_pen = QPen(Qt.black, 0.5)
-        self._link_icon_extent = 4 * self.magic_number
-        self._link_icon = _LinkIcon(0, 0, self._link_icon_extent, self._link_icon_extent, self)
-        self._link_icon.setPen(self.normal_pen)
-        self._link_icon.update_icon()
-        self.setBrush(QBrush(self._COLOR))
         self.parallel_link = None
-        self.setFlag(QGraphicsItem.ItemIsSelectable, enabled=True)
-        self.setFlag(QGraphicsItem.ItemIsFocusable, enabled=True)
         self.setZValue(0.5)  # This makes links appear on top of items because item zValue == 0.0
-        self.update_geometry()
-        self._exec_color = None
-        self.resource_filter_model = ResourceFilterModel(self._connection, toolbox.undo_stack, toolbox)
+        self.update_icons()
 
-    def refresh_resource_filter_model(self):
-        """Makes resource filter mode fetch filter data from database."""
-        self.resource_filter_model.build_tree()
-
-    @busy_effect
-    def set_connection_options(self, options):
-        if options == self.connection.options:
-            return
-        self.connection.options = options
-        self._link_icon.update_icon()
-        item = self._toolbox.project_item_model.get_item(self.connection.source).project_item
-        self._toolbox.project().notify_resource_changes_to_successors(item)
-        if self is self._toolbox.active_link:
-            self._toolbox.link_properties_widgets[Link].load_connection_options()
+    def update_icons(self):
+        while self._icons:
+            self._icons.pop(0).wipe_out()
+        if self._connection.use_datapackage:
+            self._icons.append(_SvgIcon(0, 0, self._icon_extent, self._icon_extent, self, self._DATAPACKAGE))
+        if self._connection.has_filters():
+            self._icons.append(_TextIcon(0, 0, self._icon_extent, self._icon_extent, self, self._FILTERS))
+        if self._connection.use_memory_db:
+            self._icons.append(_TextIcon(0, 0, self._icon_extent, self._icon_extent, self, self._MEMORY))
+        self._place_icons()
 
     @property
     def name(self):
@@ -390,77 +474,9 @@ class Link(LinkBase):
     def connection(self):
         return self._connection
 
-    def do_update_geometry(self, guide_path):
-        """See base class."""
-        super().do_update_geometry(guide_path)
-        center = guide_path.pointAtPercent(0.5)
-        self._link_icon.setPos(center - 0.5 * QPointF(self._link_icon_extent, self._link_icon_extent))
-
-    def make_execution_animation(self, excluded):
-        """Returns an animation to play when execution 'passes' through this link.
-
-        Returns:
-            QVariantAnimation
-        """
-        colorname = "lightGray" if excluded else "red"
-        self._exec_color = QColor(colorname)
-        qsettings = self._toolbox.qsettings()
-        duration = int(qsettings.value("appSettings/dataFlowAnimationDuration", defaultValue="100"))
-        animation = QVariantAnimation()
-        animation.setStartValue(0.0)
-        animation.setEndValue(1.0)
-        animation.setDuration(duration)
-        animation.valueChanged.connect(self._handle_execution_animation_value_changed)
-        animation.finished.connect(lambda: self.setBrush(self._COLOR))
-        animation.finished.connect(animation.deleteLater)
-        return animation
-
-    @Slot(object)
-    def _handle_execution_animation_value_changed(self, step):
-        gradient = QLinearGradient(self.src_center, self.dst_center)
-        delta = 8 * self.magic_number / QLineF(self.src_center, self.dst_center).length()
-        gradient.setColorAt(0, self._COLOR)
-        gradient.setColorAt(max(0.0, step - delta), self._COLOR)
-        gradient.setColorAt(step, self._exec_color)
-        gradient.setColorAt(min(1.0, step + delta), self._COLOR)
-        gradient.setColorAt(1.0, self._COLOR)
-        self.setBrush(gradient)
-
-    def mousePressEvent(self, e):
-        """Ignores event if there's a connector button underneath,
-        to allow creation of new links.
-
-        Args:
-            e (QGraphicsSceneMouseEvent): Mouse event
-        """
-        if any(isinstance(x, ConnectorButton) for x in self.scene().items(e.scenePos())):
-            e.ignore()
-
-    def contextMenuEvent(self, e):
-        """Selects the link and shows context menu.
-
-        Args:
-            e (QGraphicsSceneMouseEvent): Mouse event
-        """
-        self.setSelected(True)
-        self._toolbox.show_link_context_menu(e.screenPos(), self)
-
-    def paint(self, painter, option, widget=None):
-        """Sets a dashed pen if selected."""
-        if option.state & QStyle.State_Selected:
-            option.state &= ~QStyle.State_Selected
-            self.setPen(self.selected_pen)
-            self._link_icon.setPen(self.selected_pen)
-        else:
-            self.setPen(self.normal_pen)
-            self._link_icon.setPen(self.normal_pen)
-        super().paint(painter, option, widget)
-
-    def shape(self):
-        shape = super().shape()
-        if self._link_icon.isVisible():
-            shape.addEllipse(self._link_icon.sceneBoundingRect())
-        return shape
+    @property
+    def item(self):
+        return self.connection
 
     def itemChange(self, change, value):
         """Brings selected link to top."""
@@ -469,22 +485,17 @@ class Link(LinkBase):
                 if not isinstance(item, Link):
                     continue
                 item.stackBefore(self)
-            return value
         return super().itemChange(change, value)
 
-    def wipe_out(self):
-        """Removes any trace of this item from the system."""
-        self._link_icon.wipe_out()
-        self.src_connector.links.remove(self)
-        self.dst_connector.links.remove(self)
-        scene = self.scene()
-        if scene:
-            scene.removeItem(self)
 
-
-class JumpLink(LinkBase):
+class JumpLink(JumpOrLink):
     """A graphics icon to represent a jump connection between items."""
 
+    _COLOR = JUMP_COLOR
+    _ISSUE_TEXT = "\uf071"
+    _NORMAL_TEXT = "\uf2f9"
+    _ISSUE_COLOR = QColor("red")
+
     def __init__(self, toolbox, src_connector, dst_connector, jump):
         """
         Args:
@@ -495,25 +506,21 @@ class JumpLink(LinkBase):
         """
         super().__init__(toolbox, src_connector, dst_connector)
         self._jump = jump
-        self.selected_pen = QPen(Qt.black, 1, Qt.DashLine)
-        self.normal_pen = QPen(Qt.black, 0.5)
-        self._icon_extent = 2.5 * self.magic_number
-        self._icon = _JumpIcon(0, 0, self._icon_extent, self._icon_extent, self)
-        self._icon.setPen(self.normal_pen)
-        self._icon.update_icon()
-        self._color = QColor(128, 0, 255, 200)
-        brush = QBrush(self._color)
-        self.setBrush(brush)
-        self.setFlag(QGraphicsItem.ItemIsSelectable, enabled=True)
-        self.setFlag(QGraphicsItem.ItemIsFocusable, enabled=True)
         self.setZValue(0.6)
-        self.update_geometry()
-        self._exec_color = None
+        self.update_icons()
 
     @property
     def jump(self):
         return self._jump
 
+    @property
+    def item(self):
+        return self.jump
+
+    @property
+    def name(self):
+        return self._jump.name
+
     def issues(self):
         """Checks if jump is well-defined.
 
@@ -522,61 +529,21 @@ class JumpLink(LinkBase):
         """
         return self._toolbox.project().jump_issues(self.jump)
 
-    def wipe_out(self):
-        """Removes any trace of this item from the system."""
-        self.src_connector.links.remove(self)
-        self.dst_connector.links.remove(self)
-        scene = self.scene()
-        if scene:
-            scene.removeItem(self)
-
-    def do_update_geometry(self, guide_path):
-        """See base class."""
-        super().do_update_geometry(guide_path)
-        center = guide_path.pointAtPercent(0.5)
-        self._icon.setPos(center - 0.5 * QPointF(self._icon_extent, self._icon_extent))
-
-    def contextMenuEvent(self, e):
-        """Selects the link and shows context menu.
-
-        Args:
-            e (QGraphicsSceneMouseEvent): Mouse event
-        """
-        self.setSelected(True)
-        self._toolbox.show_link_context_menu(e.screenPos(), self)
-
-    def mousePressEvent(self, e):
-        """Ignores event if there's a connector button underneath,
-        to allow creation of new links.
-
-        Args:
-            e (QGraphicsSceneMouseEvent): Mouse event
-        """
-        if any(isinstance(x, ConnectorButton) for x in self.scene().items(e.scenePos())):
-            e.ignore()
-
-    def paint(self, painter, option, widget=None):
-        """Sets a dashed pen if selected."""
-        if option.state & QStyle.State_Selected:
-            option.state &= ~QStyle.State_Selected
-            self.setPen(self.selected_pen)
+    def update_icons(self):
+        while self._icons:
+            self._icons.pop(0).wipe_out()
+        issues = self.issues()
+        if issues:
+            text = self._ISSUE_TEXT
+            color = self._ISSUE_COLOR
+            tooltip = issues[0]
         else:
-            self.setPen(self.normal_pen)
-        super().paint(painter, option, widget)
-
-    def make_execution_animation(self, excluded):
-        """Returns an animation to play when execution 'passes' through this link.
-
-        Returns:
-            QVariantAnimation
-        """
-        animation = QVariantAnimation()
-        animation.finished.connect(animation.deleteLater())
-        return animation
-
-    def update_geometry(self, curved_links=None):
-        """Forces curved links."""
-        super().update_geometry(curved_links=True)
+            text = self._NORMAL_TEXT
+            color = None
+            tooltip = ""
+        icon = _TextIcon(0, 0, self._icon_extent, self._icon_extent, self, text, color=color, tooltip=tooltip)
+        self._icons.append(icon)
+        self._place_icons()
 
 
 class LinkDrawerBase(LinkBase):
@@ -589,7 +556,6 @@ class LinkDrawerBase(LinkBase):
         """
         super().__init__(toolbox, None, None)
         self.tip = None
-        self.setPen(QPen(Qt.black, 0.5))
         self.setZValue(1)  # A drawer should be on top of every other item.
 
     @property
@@ -612,6 +578,11 @@ class LinkDrawerBase(LinkBase):
         # the tip 'snap' to the center of the connector button
         return self.dst_rect.center()
 
+    def _get_dst_offset(self):
+        if self.dst_connector is None:
+            return QPointF(0, 0)
+        return super()._get_dst_offset()
+
     def add_link(self):
         """Makes link between source and destination connectors."""
         raise NotImplementedError()
@@ -623,9 +594,13 @@ class LinkDrawerBase(LinkBase):
         Args:
             src_connector (ConnectorButton): source connector
         """
-        src_connector.scene().addItem(self)
+        view = self._toolbox.ui.graphicsView
+        self.tip = view.mapToScene(view.mapFromGlobal(QCursor.pos()))
         self.src_connector = src_connector
-        self.tip = src_connector.sceneBoundingRect().center()
+        self.src_connector.scene().addItem(self)
+        self._stroker.setWidth(self.magic_number)
+        self._pen.setWidthF(self.magic_number)
+        self.setPen(self._pen)
         self.update_geometry()
         self.show()
 
@@ -642,13 +617,15 @@ class LinkDrawerBase(LinkBase):
 class ConnectionLinkDrawer(LinkDrawerBase):
     """An item for drawing connection links between project items."""
 
+    _COLOR = LINK_COLOR.lighter()
+
     def __init__(self, toolbox):
         """
         Args:
             toolbox (ToolboxUI): main UI class instance
         """
         super().__init__(toolbox)
-        self.setBrush(QBrush(QColor(255, 255, 0, 100)))
+        self._pen.setBrush(QBrush(self._COLOR))
 
     def add_link(self):
         self._toolbox.ui.graphicsView.add_link(self.src_connector, self.dst_connector)
@@ -666,13 +643,15 @@ class ConnectionLinkDrawer(LinkDrawerBase):
 class JumpLinkDrawer(LinkDrawerBase):
     """An item for drawing jump connections between project items."""
 
+    _COLOR = JUMP_COLOR.lighter()
+
     def __init__(self, toolbox):
         """
         Args:
             toolbox (ToolboxUI): main UI class instance
         """
         super().__init__(toolbox)
-        self.setBrush(QBrush(QColor(128, 0, 255, 100)))
+        self._pen.setBrush(QBrush(self._COLOR))
 
     def add_link(self):
         self._toolbox.ui.graphicsView.add_jump(self.src_connector, self.dst_connector)
