@@ -19,7 +19,7 @@ These models concatenate several 'single' models and one 'empty' model.
 from PySide2.QtCore import Qt, Slot, QTimer, QModelIndex
 from PySide2.QtGui import QFont
 from spinedb_api.parameter_value import join_value_and_type
-from ...helpers import rows_to_row_count_tuples
+from ...helpers import rows_to_row_count_tuples, parameter_identifier, FetchParent
 from ..widgets.custom_menus import ParameterViewFilterMenu
 from ...mvcmodels.compound_table_model import CompoundWithEmptyTableModel
 from .empty_parameter_models import (
@@ -36,7 +36,7 @@ from .single_parameter_models import (
 )
 
 
-class CompoundParameterModel(CompoundWithEmptyTableModel):
+class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
     """A model that concatenates several single parameter models
     and one empty parameter model.
     """
@@ -55,25 +55,27 @@ class CompoundParameterModel(CompoundWithEmptyTableModel):
         self.db_maps = db_maps
         self._filter_class_ids = {}
         self._auto_filter_menus = {}
-        self._auto_filter = dict()  # Maps field to db map, to entity id, to *accepted* item ids
+        self._auto_filter = {}  # Maps field to db map, to entity id, to *accepted* item ids
         self._filter_timer = QTimer(self)
         self._filter_timer.setSingleShot(True)
         self._filter_timer.setInterval(100)
         self._filter_timer.timeout.connect(self.refresh)
 
-    def canFetchMore(self, parent=QModelIndex()):
-        return any(self.db_mngr.can_fetch_more(db_map, self.item_type, parent=self) for db_map in self.db_maps)
+    @property
+    def fetch_item_type(self):
+        return self.item_type
 
-    def fetchMore(self, parent=QModelIndex()):
+    def canFetchMore(self, _parent):
+        return any(self.db_mngr.can_fetch_more(db_map, self) for db_map in self.db_maps)
+
+    def fetchMore(self, _parent):
         for db_map in self.db_maps:
-            self.db_mngr.fetch_more(db_map, self.item_type, parent=self)
+            self.db_mngr.fetch_more(db_map, self)
 
-    def fetch_successful(self, db_map, item):
-        model = self._create_single_model(db_map, item["entity_class_id"])
-        return self.filter_accepts_model(model) and model.filter_accepts_item(item)
-
-    def fetch_id(self):
-        return str(self.__dict__)
+    def filter_query(self, query, subquery, db_map):
+        return query.filter(getattr(subquery.c, self.entity_class_id_key).isnot(None)).order_by(
+            subquery.c.entity_class_name
+        )
 
     def _make_header(self):
         raise NotImplementedError()
@@ -153,7 +155,9 @@ class CompoundParameterModel(CompoundWithEmptyTableModel):
     def init_model(self):
         """Initializes the model."""
         super().init_model()
-        self.empty_model.fetchMore()
+        self._filter_class_ids = {}
+        self._auto_filter = {}
+        self.empty_model.fetchMore(QModelIndex())
         self._make_auto_filter_menus()
 
     def _make_auto_filter_menus(self):
@@ -207,14 +211,6 @@ class CompoundParameterModel(CompoundWithEmptyTableModel):
             return italic_font
         return super().headerData(section, orientation, role)
 
-    def _create_single_models(self):
-        """Returns a list of single models for this compound model, one for each entity_class in each database.
-
-        Returns:
-            list
-        """
-        return []
-
     def _create_empty_model(self):
         """Returns the empty model for this compound model.
 
@@ -313,8 +309,8 @@ class CompoundParameterModel(CompoundWithEmptyTableModel):
         if model.set_auto_filter(field, values):
             self._invalidate_filter()
 
-    def _row_map_for_model(self, model):
-        """Returns the row map for the given model.
+    def _row_map_iterator_for_model(self, model):
+        """Yields row map for the given model.
         Reimplemented to take filter status into account.
 
         Args:
@@ -324,8 +320,9 @@ class CompoundParameterModel(CompoundWithEmptyTableModel):
             list: tuples (model, row number) for each accepted row
         """
         if not self.filter_accepts_model(model):
-            return []
-        return [(model, i) for i in model.accepted_rows()]
+            return ()
+        for i in model.accepted_rows():
+            yield (model, i)
 
     def _models_with_db_map(self, db_map):
         """Returns a collection of single models with given db_map.
@@ -382,23 +379,42 @@ class CompoundParameterModel(CompoundWithEmptyTableModel):
         for db_map, items in db_map_data.items():
             items_per_class = self._items_per_class(items)
             for entity_class_id, class_items in items_per_class.items():
-                ids = [item["id"] for item in class_items]
-                self._create_and_append_single_model(db_map, entity_class_id, ids)
+                ids_committed = list()
+                ids_uncommitted = list()
+                for item in class_items:
+                    if item.get("commit_id") is not None:
+                        ids_committed.append(item["id"])
+                    else:
+                        ids_uncommitted.append(item["id"])
+                self._add_parameter_data(db_map, entity_class_id, ids_committed, committed=True)
+                self._add_parameter_data(db_map, entity_class_id, ids_uncommitted, committed=False)
                 self._do_add_data_to_filter_menus(db_map, class_items)
         self.empty_model.receive_parameter_data_added(db_map_data)
 
-    def _create_single_model(self, db_map, entity_class_id):
-        model = self._single_model_type(self.header, self.db_mngr, db_map, entity_class_id)
+    def _get_insert_position(self, model):
+        if model.committed:
+            return super()._get_insert_position(model)
+        return len(self.single_models)
+
+    def _create_single_model(self, db_map, entity_class_id, committed):
+        model = self._single_model_type(self.header, self.db_mngr, db_map, entity_class_id, committed)
+        self._connect_single_model(model)
         for field in self._auto_filter:
             self._set_single_auto_filter(model, field)
         return model
 
-    def _create_and_append_single_model(self, db_map, entity_class_id, ids):
-        model = self._create_single_model(db_map, entity_class_id)
+    def _add_parameter_data(self, db_map, entity_class_id, ids, committed):
+        if not ids:
+            return
+        if committed:
+            existing = next(
+                (m for m in self.single_models if (m.db_map, m.entity_class_id) == (db_map, entity_class_id)), None
+            )
+            if existing is not None:
+                existing.add_rows(ids)
+                return
+        model = self._create_single_model(db_map, entity_class_id, committed)
         model.reset_model(ids)
-        single_row_map = self._row_map_for_model(model)
-        self._insert_single_row_map(single_row_map)
-        self.sub_models.insert(len(self.single_models), model)
 
     def receive_parameter_data_updated(self, db_map_data):
         """Runs when either parameter definitions or values are updated in the dbs.
@@ -459,18 +475,30 @@ class CompoundParameterModel(CompoundWithEmptyTableModel):
         return sub_model.db_map, sub_model.item_id(sub_index.row())
 
     def index_name(self, index):
+        """Generates a name for data at given index.
+
+        Args:
+            index (QModelIndex): index to model
+
+        Returns:
+            str: label identifying the data
+        """
         item = self.db_item(index)
         if item is None:
             return ""
-        entity_name_key = {
+        database = self.index(index.row(), self.columnCount() - 1).data()
+        name_key = {
             "parameter_definition": {
                 "object_class": "object_class_name",
                 "relationship_class": "relationship_class_name",
             },
             "parameter_value": {"object_class": "object_name", "relationship_class": "object_name_list"},
         }[self.item_type][self.entity_class_type]
-        entity_name = item[entity_name_key].replace(",", self.db_mngr.GROUP_SEP)
-        return entity_name + " - " + item["parameter_name"]
+        names = item[name_key].split(",")
+        alternative_name = {"parameter_definition": lambda x: None, "parameter_value": lambda x: x["alternative_name"]}[
+            self.item_type
+        ](item)
+        return parameter_identifier(database, item["parameter_name"], names, alternative_name)
 
     def get_set_data_delayed(self, index):
         """Returns a function that ParameterValueEditor can call to set data for the given index at any later time,
@@ -543,8 +571,15 @@ class CompoundParameterDefinitionMixin:
 class CompoundParameterValueMixin:
     """Handles signals from db mngr for parameter_value models."""
 
-    _filter_entity_ids = dict()
-    _filter_alternative_ids = dict()
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._filter_entity_ids = dict()
+        self._filter_alternative_ids = dict()
+
+    def init_model(self):
+        super().init_model()
+        self._filter_entity_ids = dict()
+        self._filter_alternative_ids = dict()
 
     @property
     def item_type(self):
@@ -572,8 +607,8 @@ class CompoundParameterValueMixin:
             if model.set_filter_alternative_ids(alternative_ids):
                 self._invalidate_filter()
 
-    def _create_single_model(self, db_map, entity_class_id):
-        model = super()._create_single_model(db_map, entity_class_id)
+    def _create_single_model(self, db_map, entity_class_id, committed):
+        model = super()._create_single_model(db_map, entity_class_id, committed)
         model.set_filter_entity_ids(self._filter_entity_ids)
         model.set_filter_alternative_ids(self._filter_alternative_ids)
         return model
