@@ -15,13 +15,15 @@ Contains the GraphViewMixin class.
 :author: M. Marin (KTH)
 :date:   26.11.2018
 """
+
+import sys
 import itertools
 from time import monotonic
 from PySide2.QtCore import Slot, QTimer, QThreadPool
 from PySide2.QtWidgets import QHBoxLayout
 from spinedb_api import from_database
 from ...widgets.custom_qgraphicsscene import CustomGraphicsScene
-from ...helpers import get_save_file_name_in_last_dir
+from ...helpers import get_save_file_name_in_last_dir, ItemTypeFetchParent
 from ..graphics_items import (
     EntityItem,
     ObjectItem,
@@ -40,7 +42,7 @@ class GraphViewMixin:
 
     VERTEX_EXTENT = 64
     _ARC_WIDTH = 0.15 * VERTEX_EXTENT
-    _ARC_LENGTH_HINT = 1.5 * VERTEX_EXTENT
+    _ARC_LENGTH_HINT = 1.0 * VERTEX_EXTENT
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -52,18 +54,19 @@ class GraphViewMixin:
         layout.addWidget(self._progress_bar)
         self._persistent = False
         self._owes_graph = False
-        self.scene = None
+        self.scene = CustomGraphicsScene(self)
+        self.ui.graphicsView.setScene(self.scene)
         self.object_items = list()
         self.relationship_items = list()
         self.arc_items = list()
         self.selected_tree_inds = {}
-        self.object_ids = list()
-        self.relationship_ids = list()
+        self.db_map_object_id_sets = list()
+        self.db_map_relationship_id_sets = list()
         self.src_inds = list()
         self.dst_inds = list()
         self._adding_relationships = False
         self._pos_for_added_objects = None
-        self.added_relationship_ids = set()
+        self.added_db_map_relationship_ids = set()
         self._thread_pool = QThreadPool()
         self.layout_gens = dict()
         self._layout_gen_id = None
@@ -72,15 +75,16 @@ class GraphViewMixin:
         self._extend_graph_timer.setInterval(100)
         self._extend_graph_timer.timeout.connect(self.build_graph)
         self._extending_graph = False
+        self._object_fetch_parent = ItemTypeFetchParent("object")
+        self._relationship_fetch_parent = ItemTypeFetchParent("relationship")
 
     @Slot(bool)
     def _stop_extending_graph(self, _=False):
         self._extending_graph = False
 
     def init_models(self):
+        self.scene.clear()
         super().init_models()
-        self.scene = CustomGraphicsScene(self)
-        self.ui.graphicsView.setScene(self.scene)
 
     def connect_signals(self):
         """Connects signals."""
@@ -88,6 +92,7 @@ class GraphViewMixin:
         self.ui.treeView_object.tree_selection_changed.connect(self.rebuild_graph)
         self.ui.treeView_relationship.tree_selection_changed.connect(self.rebuild_graph)
         self.ui.dockWidget_entity_graph.visibilityChanged.connect(self._handle_entity_graph_visibility_changed)
+        self.scene.selectionChanged.connect(self.ui.graphicsView.handle_scene_selection_changed)
 
     def receive_objects_added(self, db_map_data):
         """Runs when objects are added to the db.
@@ -96,23 +101,24 @@ class GraphViewMixin:
         Args:
             db_map_data (dict): list of dictionary-items keyed by DiffDatabaseMapping instance.
         """
-        super().receive_objects_added(db_map_data)
-        added_ids = {(db_map, x["id"]) for db_map, objects in db_map_data.items() for x in objects}
-        restored_ids = self.restore_removed_entities(added_ids)
-        added_ids -= restored_ids
-        if added_ids and self._pos_for_added_objects is not None:
+        new_db_map_id_sets = self.add_db_map_ids_to_items(db_map_data, ObjectItem)
+        if not new_db_map_id_sets:
+            super().receive_objects_added(db_map_data)
+            return
+        if self._pos_for_added_objects is not None:
             spread = self.VERTEX_EXTENT * self.ui.graphicsView.zoom_factor
-            gen = GraphLayoutGenerator(None, len(added_ids), spread=spread)
+            gen = GraphLayoutGenerator(None, len(new_db_map_id_sets), spread=spread)
             gen.run()
             x = self._pos_for_added_objects.x()
             y = self._pos_for_added_objects.y()
-            for dx, dy, object_id in zip(gen.x, gen.y, added_ids):
-                object_item = ObjectItem(self, x + dx, y + dy, self.VERTEX_EXTENT, object_id)
+            for dx, dy, db_map_ids in zip(gen.x, gen.y, new_db_map_id_sets):
+                object_item = ObjectItem(self, x + dx, y + dy, self.VERTEX_EXTENT, tuple(db_map_ids))
                 self.scene.addItem(object_item)
                 object_item.apply_zoom(self.ui.graphicsView.zoom_factor)
             self._pos_for_added_objects = None
         elif self._extending_graph:
             self._extend_graph_timer.start()
+        super().receive_objects_added(db_map_data)
 
     def receive_relationships_added(self, db_map_data):
         """Runs when relationships are added to the db.
@@ -121,24 +127,26 @@ class GraphViewMixin:
         Args:
             db_map_data (dict): list of dictionary-items keyed by DiffDatabaseMapping instance.
         """
-        super().receive_relationships_added(db_map_data)
-        added_ids = {(db_map, x["id"]) for db_map, relationships in db_map_data.items() for x in relationships}
-        restored_ids = self.restore_removed_entities(added_ids)
-        added_ids -= restored_ids
-        if added_ids and self._adding_relationships:
-            self.added_relationship_ids.update(added_ids)
+        new_db_map_id_sets = self.add_db_map_ids_to_items(db_map_data, RelationshipItem)
+        if not new_db_map_id_sets:
+            super().receive_relationships_added(db_map_data)
+            return
+        if self._adding_relationships:
+            for db_map_ids in new_db_map_id_sets:
+                self.added_db_map_relationship_ids.update(db_map_ids)
             self.build_graph(persistent=True)
             self._end_add_relationships()
         elif self._extending_graph:
             self._extend_graph_timer.start()
+        super().receive_relationships_added(db_map_data)
 
     def receive_object_classes_updated(self, db_map_data):
-        super().receive_object_classes_updated(db_map_data)
         self.refresh_icons(db_map_data)
+        super().receive_object_classes_updated(db_map_data)
 
     def receive_relationship_classes_updated(self, db_map_data):
-        super().receive_relationship_classes_updated(db_map_data)
         self.refresh_icons(db_map_data)
+        super().receive_relationship_classes_updated(db_map_data)
 
     def receive_objects_updated(self, db_map_data):
         """Runs when objects are updated in the db. Refreshes names of objects in graph.
@@ -146,12 +154,11 @@ class GraphViewMixin:
         Args:
             db_map_data (dict): list of dictionary-items keyed by DiffDatabaseMapping instance.
         """
-        super().receive_objects_updated(db_map_data)
-        updated_ids = {(db_map, x["id"]): x["name"] for db_map, objects in db_map_data.items() for x in objects}
         for item in self.ui.graphicsView.items():
-            if isinstance(item, ObjectItem) and item.db_map_entity_id in updated_ids:
-                name = updated_ids[item.db_map_entity_id]
-                item.update_name(name)
+            if isinstance(item, ObjectItem) and not item.update_name():
+                self.build_graph(persistent=True)
+                break
+        super().receive_objects_updated(db_map_data)
 
     def receive_objects_removed(self, db_map_data):
         """Runs when objects are removed from the db. Rebuilds graph if needed.
@@ -159,8 +166,21 @@ class GraphViewMixin:
         Args:
             db_map_data (dict): list of dictionary-items keyed by DiffDatabaseMapping instance.
         """
+        self.hide_removed_entities(db_map_data, ObjectItem)
         super().receive_objects_removed(db_map_data)
-        self.hide_removed_entities(db_map_data)
+
+    def receive_relationships_updated(self, db_map_data):
+        """Runs when relationships are updated in the db.
+
+        Args:
+            db_map_data (dict): list of dictionary-items keyed by DiffDatabaseMapping instance.
+        """
+        updated_ids = {(db_map, x["id"]) for db_map, rels in db_map_data.items() for x in rels}
+        for item in self.ui.graphicsView.items():
+            if isinstance(item, RelationshipItem) and set(item.db_map_ids).intersection(updated_ids):
+                self.build_graph(persistent=True)
+                break
+        super().receive_relationships_updated(db_map_data)
 
     def receive_relationships_removed(self, db_map_data):
         """Runs when relationships are removed from the db. Rebuilds graph if needed.
@@ -168,42 +188,64 @@ class GraphViewMixin:
         Args:
             db_map_data (dict): list of dictionary-items keyed by DiffDatabaseMapping instance.
         """
+        self.hide_removed_entities(db_map_data, RelationshipItem)
         super().receive_relationships_removed(db_map_data)
-        self.hide_removed_entities(db_map_data)
 
-    def restore_removed_entities(self, added_ids):
-        """Restores any entities that have been previously removed and returns their ids.
-        This happens in the context of undo/redo.
+    def add_db_map_ids_to_items(self, db_map_data, type_):
+        """Goes through items of given type and adds the corresponding db_map ids.
+        This could mean either restoring removed (db_map, id) tuples previously removed,
+        or adding new (db_map, id) tuples.
 
         Args:
-            added_ids (set(int)): Set of newly added ids.
+            db_map_data (dict(DiffDatabaseMapping, list)): List of added items keyed by db_map
 
         Returns:
-            set(int)
+            list: tuples (db_map, id) that didn't match any item in the view.
         """
-        restored_items = [item for item in self.ui.graphicsView.removed_items if item.db_map_entity_id in added_ids]
+        get_key = {ObjectItem: self._get_object_key, RelationshipItem: self._get_relationship_key}[type_]
+        added_db_map_ids_by_key = {}
+        for db_map, entities in db_map_data.items():
+            for entity in entities:
+                db_map_id = (db_map, entity["id"])
+                key = get_key(db_map_id)
+                added_db_map_ids_by_key.setdefault(key, set()).add(db_map_id)
+        restored_items = set()
+        for item in self.ui.graphicsView.items():
+            if not isinstance(item, type_):
+                continue
+            for db_map_id in item.original_db_map_ids:
+                try:
+                    key = get_key(db_map_id)
+                except KeyError:
+                    continue
+                db_map_ids = added_db_map_ids_by_key.pop(key, None)
+                if db_map_ids:
+                    item.add_db_map_ids(db_map_ids)
+                    restored_items.add(item)
         for item in restored_items:
-            self.ui.graphicsView.removed_items.remove(item)
-            item.set_all_visible(True)
-        return {item.db_map_entity_id for item in restored_items}
+            self.ui.graphicsView.removed_items.discard(item)
+            item.setVisible(True)
+        return list(added_db_map_ids_by_key.values())
 
-    def hide_removed_entities(self, db_map_data):
-        """Hides removed entities while saving them into a list attribute.
+    def hide_removed_entities(self, db_map_data, type_):
+        """Hides removed entities while saving them into a set.
         This allows entities to be restored in case the user undoes the operation."""
-        removed_ids = {(db_map, x["id"]) for db_map, items in db_map_data.items() for x in items}
-        self.added_relationship_ids -= removed_ids
-        removed_items = [
-            item
-            for item in self.ui.graphicsView.items()
-            if isinstance(item, EntityItem) and item.db_map_entity_id in removed_ids
-        ]
+        removed_db_map_ids = {(db_map, x["id"]) for db_map, items in db_map_data.items() for x in items}
+        self.added_db_map_relationship_ids -= removed_db_map_ids
+        removed_items = set()
+        for item in self.ui.graphicsView.items():
+            if not isinstance(item, type_):
+                continue
+            item.remove_db_map_ids(removed_db_map_ids)
+            if not item.db_map_ids:
+                removed_items.add(item)
         if not removed_items:
             return
-        self.ui.graphicsView.removed_items.extend(removed_items)
+        self.ui.graphicsView.removed_items |= removed_items
         scene = self.scene
         self.scene = None
         for item in removed_items:
-            item.set_all_visible(False)
+            item.setVisible(False)
         self.scene = scene
 
     def refresh_icons(self, db_map_data):
@@ -230,7 +272,7 @@ class GraphViewMixin:
         """Stores the given selection of entity tree indexes and builds graph."""
         if selected is not None:
             self.selected_tree_inds = selected
-        self.added_relationship_ids.clear()
+        self.added_db_map_relationship_ids.clear()
         self._extending_graph = True
         self.build_graph()
 
@@ -254,7 +296,7 @@ class GraphViewMixin:
         self._progress_bar.set_layout_generator(layout_gen)
         self._progress_bar.show()
         layout_gen.layout_available.connect(self._complete_graph)
-        layout_gen.finished.connect(lambda id_: self.layout_gens.pop(id_))  # Lambda to avoid issues in Python 3.7
+        layout_gen.finished.connect(lambda id_: self.layout_gens.pop(id_, None))  # Lambda to avoid issues in Python 3.7
         self._thread_pool.start(layout_gen)
 
     def _stop_layout_generators(self):
@@ -292,9 +334,9 @@ class GraphViewMixin:
         """
         if "root" in self.selected_tree_inds:
             for db_map in self.db_maps:
-                for item_type in ("object", "relationship"):
-                    if self.db_mngr.can_fetch_more(db_map, item_type):
-                        self.db_mngr.fetch_more(db_map, item_type)
+                for fetch_parent in (self._object_fetch_parent, self._relationship_fetch_parent):
+                    if self.db_mngr.can_fetch_more(db_map, fetch_parent):
+                        self.db_mngr.fetch_more(db_map, fetch_parent)
             return (
                 set((db_map, x["id"]) for db_map in self.db_maps for x in self.db_mngr.get_items(db_map, "object")),
                 set(),
@@ -312,69 +354,109 @@ class GraphViewMixin:
         for index in self.selected_tree_inds.get("object_class", {}):
             item = index.model().item_from_index(index)
             item.fetch_more_if_possible()
-            for db_map in item.db_maps:
-                object_ids = set((db_map, id_) for id_ in item.get_children_ids(db_map))
-                selected_object_ids.update(object_ids)
+            selected_object_ids.update(item.get_children_ids())
         for index in self.selected_tree_inds.get("relationship_class", {}):
             item = index.model().item_from_index(index)
             item.fetch_more_if_possible()
-            for db_map in item.db_maps:
-                relationship_ids = set((db_map, id_) for id_ in item.get_children_ids(db_map))
-                selected_relationship_ids.update(relationship_ids)
+            selected_relationship_ids.update(item.get_children_ids())
         return selected_object_ids, selected_relationship_ids
 
-    def _get_all_relationships_for_graph(self, object_ids, relationship_ids):
+    def _get_db_map_relationships_for_graph(self, db_map_object_ids, db_map_relationship_ids):
         cond = any if self.ui.graphicsView.auto_expand_objects else all
         return [
             (db_map, x)
             for db_map in self.db_maps
-            for x in self.db_mngr.get_items(db_map, "relationship")
-            if cond([(db_map, int(id_)) in object_ids for id_ in x["object_id_list"].split(",")])
-        ] + [(db_map, self.db_mngr.get_item(db_map, "relationship", id_)) for db_map, id_ in relationship_ids]
+            for x in self.db_mngr.get_items(db_map, "relationship", only_visible=False)
+            if cond([(db_map, int(id_)) in db_map_object_ids for id_ in x["object_id_list"].split(",")])
+        ] + [(db_map, self.db_mngr.get_item(db_map, "relationship", id_)) for db_map, id_ in db_map_relationship_ids]
 
     def _update_graph_data(self):
         """Updates data for graph according to selection in trees."""
-        object_ids, relationship_ids = self._get_selected_entity_ids()
-        relationship_ids.update(self.added_relationship_ids)
-        pruned_entity_ids = {id_ for ids in self.ui.graphicsView.pruned_entity_ids.values() for id_ in ids}
-        object_ids -= pruned_entity_ids
-        relationship_ids -= pruned_entity_ids
-        relationships = self._get_all_relationships_for_graph(object_ids, relationship_ids)
-        object_id_lists = dict()
-        for db_map, relationship in relationships:
-            if (db_map, relationship["id"]) in pruned_entity_ids:
+        db_map_object_ids, db_map_relationship_ids = self._get_selected_entity_ids()
+        db_map_relationship_ids.update(self.added_db_map_relationship_ids)
+        pruned_db_map_entity_ids = {
+            id_ for ids in self.ui.graphicsView.pruned_db_map_entity_ids.values() for id_ in ids
+        }
+        db_map_object_ids -= pruned_db_map_entity_ids
+        db_map_relationship_ids -= pruned_db_map_entity_ids
+        db_map_relationships = self._get_db_map_relationships_for_graph(db_map_object_ids, db_map_relationship_ids)
+        db_map_object_id_lists = dict()
+        max_rel_dim = (
+            self.ui.graphicsView.max_relationship_dimension
+            if not self.ui.graphicsView.disable_max_relationship_dimension
+            else sys.maxsize
+        )
+        for db_map, relationship in db_map_relationships:
+            if (db_map, relationship["id"]) in pruned_db_map_entity_ids:
                 continue
-            object_id_list = [
+            db_map_object_id_list = [
                 (db_map, id_)
                 for id_ in (int(x) for x in relationship["object_id_list"].split(","))
-                if (db_map, id_) not in pruned_entity_ids
+                if (db_map, id_) not in pruned_db_map_entity_ids
             ]
-            if len(object_id_list) < 2:
+            if len(db_map_object_id_list) < 2 or len(db_map_object_id_list) > max_rel_dim:
                 continue
-            object_ids.update(object_id_list)
-            object_id_lists[db_map, relationship["id"]] = object_id_list
-        self.object_ids = list(object_ids)
-        self.relationship_ids = list(object_id_lists)
-        self._update_src_dst_inds(object_id_lists)
+            db_map_object_ids.update(db_map_object_id_list)
+            db_map_object_id_lists[db_map, relationship["id"]] = db_map_object_id_list
+        db_map_object_ids_by_key = {}
+        db_map_relationship_ids_by_key = {}
+        # NOTE: To have a different item for each database, we just need to include the db in the key computation below
+        for db_map_object_id in db_map_object_ids:
+            key = self._get_object_key(db_map_object_id)
+            db_map_object_ids_by_key.setdefault(key, set()).add(db_map_object_id)
+        for db_map_relationship_id in db_map_object_id_lists:
+            key = self._get_relationship_key(db_map_relationship_id)
+            db_map_relationship_ids_by_key.setdefault(key, set()).add(db_map_relationship_id)
+        self.db_map_object_id_sets = list(db_map_object_ids_by_key.values())
+        self.db_map_relationship_id_sets = list(db_map_relationship_ids_by_key.values())
+        self._update_src_dst_inds(db_map_object_id_lists)
 
-    def _update_src_dst_inds(self, object_id_lists):
+    def _get_object_key(self, db_map_object_id):
+        db_map, object_id = db_map_object_id
+        object_ = self.db_mngr.get_item(db_map, "object", object_id)
+        key = (object_["class_name"], object_["name"])
+        if not self.ui.graphicsView.merge_dbs:
+            key += (db_map.codename,)
+        return key
+
+    def _get_relationship_key(self, db_map_relationship_id):
+        db_map, relationship_id = db_map_relationship_id
+        relationship = self.db_mngr.get_item(db_map, "relationship", relationship_id)
+        key = (relationship["class_name"], relationship["object_class_name_list"], relationship["object_name_list"])
+        if not self.ui.graphicsView.merge_dbs:
+            key += (db_map.codename,)
+        return key
+
+    def _update_src_dst_inds(self, db_map_object_id_lists):
         self.src_inds = list()
         self.dst_inds = list()
-        object_ind_lookup = {id_: k for k, id_ in enumerate(self.object_ids)}
-        relationship_ind_lookup = {id_: len(self.object_ids) + k for k, id_ in enumerate(self.relationship_ids)}
-        for relationship_id, object_id_list in object_id_lists.items():
-            object_inds = [object_ind_lookup[object_id] for object_id in object_id_list]
-            relationship_ind = relationship_ind_lookup[relationship_id]
+        obj_ind_lookup = {
+            db_map_obj_id: k
+            for k, db_map_obj_ids in enumerate(self.db_map_object_id_sets)
+            for db_map_obj_id in db_map_obj_ids
+        }
+        rel_ind_lookup = {
+            db_map_rel_id: len(self.db_map_object_id_sets) + k
+            for k, db_map_rel_ids in enumerate(self.db_map_relationship_id_sets)
+            for db_map_rel_id in db_map_rel_ids
+        }
+        edges = dict()
+        for db_map_rel_id, db_map_object_id_list in db_map_object_id_lists.items():
+            object_inds = [obj_ind_lookup[db_map_obj_id] for db_map_obj_id in db_map_object_id_list]
+            relationship_ind = rel_ind_lookup[db_map_rel_id]
             for object_ind in object_inds:
-                self.src_inds.append(relationship_ind)
-                self.dst_inds.append(object_ind)
+                edges[relationship_ind, object_ind] = None
+        for src, dst in edges:  # pylint: disable=dict-iter-missing-items
+            self.src_inds.append(src)
+            self.dst_inds.append(dst)
 
     def _get_parameter_positions(self, parameter_name):
-        # FIXME: We might need to fetch all parameter values here!!!
         if not parameter_name:
             yield from []
         for db_map in self.db_maps:
-            for p in self.db_mngr.get_items_by_field(db_map, "parameter_value", "parameter_name", parameter_name):
+            for p in self.db_mngr.get_items_by_field(
+                db_map, "parameter_value", "parameter_name", parameter_name, only_visible=False
+            ):
                 pos = from_database(p["value"], p["type"])
                 if isinstance(pos, float):
                     yield (db_map, p["entity_id"]), pos
@@ -389,16 +471,21 @@ class GraphViewMixin:
         if self._persistent:
             for item in self.ui.graphicsView.items():
                 if isinstance(item, EntityItem):
-                    fixed_positions[item.db_map_entity_id] = {"x": item.pos().x(), "y": item.pos().y()}
+                    fixed_positions[item.first_db_map, item.first_id] = {"x": item.pos().x(), "y": item.pos().y()}
         param_pos_x = dict(self._get_parameter_positions(self.ui.graphicsView.pos_x_parameter))
         param_pos_y = dict(self._get_parameter_positions(self.ui.graphicsView.pos_y_parameter))
         for db_map_entity_id in param_pos_x.keys() & param_pos_y.keys():
             fixed_positions[db_map_entity_id] = {"x": param_pos_x[db_map_entity_id], "y": param_pos_y[db_map_entity_id]}
-        entity_ids = self.object_ids + self.relationship_ids
-        heavy_positions = {ind: fixed_positions[id_] for ind, id_ in enumerate(entity_ids) if id_ in fixed_positions}
+        db_map_entity_ids = self.db_map_object_id_sets + self.db_map_relationship_id_sets
+        heavy_positions = {
+            ind: fixed_positions[db_map_entity_id]
+            for ind, db_map_entity_ids in enumerate(db_map_entity_ids)
+            for db_map_entity_id in db_map_entity_ids
+            if db_map_entity_id in fixed_positions
+        }
         return GraphLayoutGenerator(
             self._layout_gen_id,
-            len(entity_ids),
+            len(db_map_entity_ids),
             self.src_inds,
             self.dst_inds,
             self._ARC_LENGTH_HINT,
@@ -412,43 +499,45 @@ class GraphViewMixin:
             x (list)
             y (list)
         """
-        self.object_items = list()
-        self.relationship_items = list()
-        self.arc_items = list()
-        for i, object_id in enumerate(self.object_ids):
-            object_item = ObjectItem(self, x[i], y[i], self.VERTEX_EXTENT, object_id)
-            self.object_items.append(object_item)
+        self.object_items = [
+            ObjectItem(self, x[i], y[i], self.VERTEX_EXTENT, tuple(db_map_object_ids))
+            for i, db_map_object_ids in enumerate(self.db_map_object_id_sets)
+        ]
         offset = len(self.object_items)
-        for i, relationship_id in enumerate(self.relationship_ids):
-            relationship_item = RelationshipItem(
-                self, x[offset + i], y[offset + i], 0.5 * self.VERTEX_EXTENT, relationship_id
+        self.relationship_items = [
+            RelationshipItem(
+                self, x[offset + i], y[offset + i], 0.5 * self.VERTEX_EXTENT, tuple(db_map_relationship_id)
             )
-            self.relationship_items.append(relationship_item)
-        for rel_ind, obj_ind in zip(self.src_inds, self.dst_inds):
-            arc_item = ArcItem(self.relationship_items[rel_ind - offset], self.object_items[obj_ind], self._ARC_WIDTH)
-            self.arc_items.append(arc_item)
+            for i, db_map_relationship_id in enumerate(self.db_map_relationship_id_sets)
+        ]
+        self.arc_items = [
+            ArcItem(self.relationship_items[rel_ind - offset], self.object_items[obj_ind], self._ARC_WIDTH)
+            for rel_ind, obj_ind in zip(self.src_inds, self.dst_inds)
+        ]
         return any(self.object_items)
 
     def _add_new_items(self):
         for item in self.object_items + self.relationship_items + self.arc_items:
             self.scene.addItem(item)
 
-    def start_relationship(self, relationship_class, obj_item):
+    def start_relationship(self, db_map, relationship_class, obj_item):
         """Starts a relationship from the given object item.
 
         Args:
+            db_map (DiffDatabaseMapping)
             relationship_class (dict)
             obj_item (..graphics_items.ObjectItem)
         """
-        db_map = obj_item.db_map
         object_class_ids_to_go = relationship_class["object_class_id_list"].copy()
-        object_class_ids_to_go.remove(obj_item.entity_class_id)
+        object_class_ids_to_go.remove(obj_item.entity_class_id(db_map))
         relationship_class["object_class_ids_to_go"] = object_class_ids_to_go
+        relationship_class["db_map"] = db_map
+        db_map_ids = ((db_map, None),)
         ch_item = CrossHairsItem(
-            self, obj_item.pos().x(), obj_item.pos().y(), 0.8 * self.VERTEX_EXTENT, db_map_entity_id=(db_map, None)
+            self, obj_item.pos().x(), obj_item.pos().y(), 0.8 * self.VERTEX_EXTENT, db_map_ids=db_map_ids
         )
         ch_rel_item = CrossHairsRelationshipItem(
-            self, obj_item.pos().x(), obj_item.pos().y(), 0.5 * self.VERTEX_EXTENT, db_map_entity_id=(db_map, None)
+            self, obj_item.pos().x(), obj_item.pos().y(), 0.5 * self.VERTEX_EXTENT, db_map_ids=db_map_ids
         )
         ch_arc_item1 = CrossHairsArcItem(ch_rel_item, obj_item, self._ARC_WIDTH)
         ch_arc_item2 = CrossHairsArcItem(ch_rel_item, ch_item, self._ARC_WIDTH)
@@ -464,11 +553,11 @@ class GraphViewMixin:
             relationship_class (dict)
             object_items (..graphics_items.ObjectItem)
         """
-        db_map = object_items[0].db_map
+        db_map = relationship_class["db_map"]
         relationships = set()
         object_class_id_list = relationship_class["object_class_id_list"]
         for item_permutation in itertools.permutations(object_items):
-            if [item.entity_class_id for item in item_permutation] == object_class_id_list:
+            if [item.entity_class_id(db_map) for item in item_permutation] == object_class_id_list:
                 relationship = tuple(item.entity_name for item in item_permutation)
                 relationships.add(relationship)
         dialog = AddReadyRelationshipsDialog(self, relationship_class, list(relationships), self.db_mngr, db_map)
@@ -502,5 +591,6 @@ class GraphViewMixin:
             event (QCloseEvent): Closing event
         """
         super().closeEvent(event)
-        self.scene.deleteLater()
+        if self.scene is not None:
+            self.scene.deleteLater()
         self.scene = None
