@@ -24,7 +24,7 @@ from PySide2.QtGui import QIcon
 from spinedb_api.helpers import fix_name_ambiguity
 from .custom_menus import TabularViewFilterMenu
 from .tabular_view_header_widget import TabularViewHeaderWidget
-from ...helpers import busy_effect, CharIconEngine
+from ...helpers import busy_effect, CharIconEngine, preferred_row_height
 from ..mvcmodels.pivot_table_models import (
     PivotTableSortFilterProxy,
     ParameterValuePivotTableModel,
@@ -34,9 +34,11 @@ from ..mvcmodels.pivot_table_models import (
 )
 from ..mvcmodels.frozen_table_model import FrozenTableModel
 
+# FIXME: only_visible=False???
+
 
 class TabularViewMixin:
-    """Provides the pivot table and its frozen table for the DS form."""
+    """Provides the pivot table and its frozen table for the Database editor."""
 
     _PARAMETER_VALUE = "&Value"
     _INDEX_EXPANSION = "&Index"
@@ -65,7 +67,7 @@ class TabularViewMixin:
         self.ui.pivot_table.setModel(self.pivot_table_proxy)
         self.ui.pivot_table.connect_spine_db_editor(self)
         self.ui.frozen_table.setModel(self.frozen_table_model)
-        self.ui.frozen_table.verticalHeader().setDefaultSectionSize(self.default_row_height)
+        self.ui.frozen_table.verticalHeader().setDefaultSectionSize(preferred_row_height(self))
 
     def populate_pivot_action_group(self):
         self.pivot_actions = {
@@ -83,14 +85,19 @@ class TabularViewMixin:
     def connect_signals(self):
         """Connects signals to slots."""
         super().connect_signals()
-        self.ui.pivot_table.horizontalHeader().header_dropped.connect(self.handle_header_dropped)
-        self.ui.pivot_table.verticalHeader().header_dropped.connect(self.handle_header_dropped)
+        self.ui.pivot_table.header_changed.connect(self._connect_pivot_table_header_signals)
         self.ui.frozen_table.header_dropped.connect(self.handle_header_dropped)
         self.ui.frozen_table.selectionModel().currentChanged.connect(self.change_frozen_value)
         self.pivot_action_group.triggered.connect(self._handle_pivot_action_triggered)
         self.ui.dockWidget_pivot_table.visibilityChanged.connect(self._handle_pivot_table_visibility_changed)
         self.ui.dockWidget_frozen_table.visibilityChanged.connect(self._handle_frozen_table_visibility_changed)
 
+    @Slot()
+    def _connect_pivot_table_header_signals(self):
+        """Connects signals of pivot table's header views."""
+        self.ui.pivot_table.horizontalHeader().header_dropped.connect(self.handle_header_dropped)
+        self.ui.pivot_table.verticalHeader().header_dropped.connect(self.handle_header_dropped)
+
     def init_models(self):
         """Initializes models."""
         super().init_models()
@@ -523,6 +530,7 @@ class TabularViewMixin:
         if self.pivot_table_model:
             self.pivot_table_model.clear_model()
             self.pivot_table_proxy.clear_filter()
+            self.pivot_table_model = None
         if self.frozen_table_model:
             self.frozen_table_model.clear_model()
 
@@ -549,6 +557,8 @@ class TabularViewMixin:
 
     @Slot()
     def _resize_pivot_header_columns(self):
+        if not self.pivot_table_model:
+            return
         top_indexes, _ = self.pivot_table_model.top_left_indexes()
         for index in top_indexes:
             self.ui.pivot_table.resizeColumnToContents(index.column())
@@ -669,8 +679,7 @@ class TabularViewMixin:
 
     @Slot("QModelIndex", "QModelIndex")
     def change_frozen_value(self, current, previous):
-        """Sets the frozen value from selection in frozen table.
-        """
+        """Sets the frozen value from selection in frozen table."""
         frozen_value = self.get_frozen_value(current)
         self.pivot_table_model.set_frozen_value(frozen_value)
         # store pivot preferences
@@ -811,124 +820,122 @@ class TabularViewMixin:
 
     def receive_alternatives_added(self, db_map_data):
         """Reacts to alternatives added event."""
-        super().receive_alternatives_added(db_map_data)
         self.receive_alternatives_added_or_removed(db_map_data, action="add")
+        super().receive_alternatives_added(db_map_data)
 
     def receive_scenarios_added(self, db_map_data):
         """Reacts to scenarios added event."""
-        super().receive_scenarios_added(db_map_data)
         self.receive_scenarios_added_or_removed(db_map_data, action="add")
+        super().receive_scenarios_added(db_map_data)
 
     def receive_objects_added(self, db_map_data):
         """Reacts to objects added event."""
-        super().receive_objects_added(db_map_data)
         self.receive_objects_added_or_removed(db_map_data, action="add")
+        super().receive_objects_added(db_map_data)
 
     def receive_relationships_added(self, db_map_data):
         """Reacts to relationships added event."""
-        super().receive_relationships_added(db_map_data)
         self.receive_relationships_added_or_removed(db_map_data, action="add")
+        super().receive_relationships_added(db_map_data)
 
     def receive_parameter_definitions_added(self, db_map_data):
         """Reacts to parameter definitions added event."""
-        super().receive_parameter_definitions_added(db_map_data)
         self.receive_parameter_definitions_added_or_removed(db_map_data, action="add")
+        super().receive_parameter_definitions_added(db_map_data)
 
     def receive_parameter_values_added(self, db_map_data):
         """Reacts to parameter values added event."""
-        super().receive_parameter_values_added(db_map_data)
         self.receive_parameter_values_added_or_removed(db_map_data, action="add")
+        super().receive_parameter_values_added(db_map_data)
 
     def receive_alternatives_updated(self, db_map_data):
         """Reacts to alternatives updated event."""
+        if self.pivot_table_model:
+            self.refresh_table_view(self.ui.pivot_table)
+            self.refresh_table_view(self.ui.frozen_table)
+            self.make_pivot_headers()
         super().receive_alternatives_updated(db_map_data)
-        if not self.pivot_table_model:
-            return
-        self.refresh_table_view(self.ui.pivot_table)
-        self.refresh_table_view(self.ui.frozen_table)
-        self.make_pivot_headers()
 
     def receive_object_classes_updated(self, db_map_data):
         """Reacts to object classes updated event."""
-        super().receive_object_classes_updated(db_map_data)
         self.receive_classes_updated(db_map_data)
+        super().receive_object_classes_updated(db_map_data)
 
     def receive_relationship_classes_updated(self, db_map_data):
         """Reacts to relationship classes updated event."""
-        super().receive_relationship_classes_updated(db_map_data)
         self.receive_classes_updated(db_map_data)
+        super().receive_relationship_classes_updated(db_map_data)
 
     def receive_objects_updated(self, db_map_data):
         """Reacts to objects updated event."""
-        super().receive_objects_updated(db_map_data)
         self.receive_db_map_data_updated(db_map_data, get_class_id=lambda x: x["class_id"])
+        super().receive_objects_updated(db_map_data)
 
     def receive_relationships_updated(self, db_map_data):
         """Reacts to relationships updated event."""
-        super().receive_relationships_updated(db_map_data)
         self.receive_db_map_data_updated(db_map_data, get_class_id=lambda x: x["class_id"])
+        super().receive_relationships_updated(db_map_data)
 
     def receive_parameter_values_updated(self, db_map_data):
         """Reacts to parameter values added event."""
-        super().receive_parameter_values_updated(db_map_data)
         self.receive_db_map_data_updated(
             db_map_data, get_class_id=lambda x: x.get("object_class_id") or x.get("relationship_class_id")
         )
+        super().receive_parameter_values_updated(db_map_data)
 
     def receive_parameter_definitions_updated(self, db_map_data):
         """Reacts to parameter definitions updated event."""
-        super().receive_parameter_definitions_updated(db_map_data)
         self.receive_db_map_data_updated(
             db_map_data, get_class_id=lambda x: x.get("object_class_id") or x.get("relationship_class_id")
         )
+        super().receive_parameter_definitions_updated(db_map_data)
 
     def receive_scenarios_updated(self, db_map_data):
+        if self.pivot_table_model:
+            self.pivot_table_model.receive_scenarios_updated(db_map_data)
         super().receive_scenarios_updated(db_map_data)
-        if not self.pivot_table_model:
-            return
-        self.pivot_table_model.receive_scenarios_updated(db_map_data)
 
     def receive_alternatives_removed(self, db_map_data):
         """Reacts to alternatives removed event."""
-        super().receive_alternatives_removed(db_map_data)
         self.receive_alternatives_added_or_removed(db_map_data, action="remove")
+        super().receive_alternatives_removed(db_map_data)
 
     def receive_scenarios_removed(self, db_map_data):
         """Reacts to scenarios removed event."""
-        super().receive_scenarios_removed(db_map_data)
         self.receive_scenarios_added_or_removed(db_map_data, action="remove")
+        super().receive_scenarios_removed(db_map_data)
 
     def receive_object_classes_removed(self, db_map_data):
         """Reacts to object classes removed event."""
-        super().receive_object_classes_removed(db_map_data)
         self.receive_classes_removed(db_map_data)
+        super().receive_object_classes_removed(db_map_data)
 
     def receive_relationship_classes_removed(self, db_map_data):
         """Reacts to relationship classes remove event."""
-        super().receive_relationship_classes_removed(db_map_data)
         self.receive_classes_removed(db_map_data)
+        super().receive_relationship_classes_removed(db_map_data)
 
     def receive_objects_removed(self, db_map_data):
         """Reacts to objects removed event."""
-        super().receive_objects_removed(db_map_data)
         self.receive_objects_added_or_removed(db_map_data, action="remove")
+        super().receive_objects_removed(db_map_data)
 
     def receive_relationships_removed(self, db_map_data):
         """Reacts to relationships removed event."""
-        super().receive_relationships_removed(db_map_data)
         self.receive_relationships_added_or_removed(db_map_data, action="remove")
+        super().receive_relationships_removed(db_map_data)
 
     def receive_parameter_definitions_removed(self, db_map_data):
         """Reacts to parameter definitions removed event."""
-        super().receive_parameter_definitions_removed(db_map_data)
         self.receive_parameter_definitions_added_or_removed(db_map_data, action="remove")
+        super().receive_parameter_definitions_removed(db_map_data)
 
     def receive_parameter_values_removed(self, db_map_data):
         """Reacts to parameter values removed event."""
-        super().receive_parameter_values_removed(db_map_data)
         self.receive_parameter_values_added_or_removed(db_map_data, action="remove")
+        super().receive_parameter_values_removed(db_map_data)
 
     def receive_session_rolled_back(self, db_maps):
         """Reacts to session rolled back event."""
-        super().receive_session_rolled_back(db_maps)
         self.clear_pivot_table()
+        super().receive_session_rolled_back(db_maps)
