@@ -32,6 +32,26 @@ class SpineDBSignaller(QObject):
         super().__init__()
         self.db_mngr = db_mngr
         self.listeners = dict()
+        self._deferred_notifications = dict()
+
+    def pause(self, *db_maps):
+        """Defers notifications from db_maps until ``resume`` is called."""
+        for db_map in db_maps:
+            self._deferred_notifications.setdefault(db_map, [])
+
+    def resume(self, *db_maps):
+        """Performs deferred notifications."""
+        for db_map in db_maps:
+            for data, method in self._deferred_notifications.pop(db_map, []):
+                method({db_map: data})
+
+    def _defer_notification(self, db_map, data, method):
+        """Defer notification if db_map is paused, and returns True. Otherwise returns False."""
+        deferred = self._deferred_notifications.get(db_map)
+        if deferred is None:
+            return False
+        deferred.append((data, method))
+        return True
 
     def add_db_map_listener(self, db_map, listener):
         """Adds listener for given db_map."""
@@ -60,10 +80,14 @@ class SpineDBSignaller(QObject):
         self.db_mngr.parameter_definitions_added.connect(self.receive_parameter_definitions_added)
         self.db_mngr.parameter_values_added.connect(self.receive_parameter_values_added)
         self.db_mngr.parameter_value_lists_added.connect(self.receive_parameter_value_lists_added)
+        self.db_mngr.list_values_added.connect(self.receive_list_values_added)
         self.db_mngr.features_added.connect(self.receive_features_added)
         self.db_mngr.tools_added.connect(self.receive_tools_added)
         self.db_mngr.tool_features_added.connect(self.receive_tool_features_added)
         self.db_mngr.tool_feature_methods_added.connect(self.receive_tool_feature_methods_added)
+        self.db_mngr.metadata_added.connect(self.receive_metadata_added)
+        self.db_mngr.entity_metadata_added.connect(self.receive_entity_metadata_added)
+        self.db_mngr.parameter_value_metadata_added.connect(self.receive_parameter_value_metadata_added)
         # Updated
         self.db_mngr.scenarios_updated.connect(self.receive_scenarios_updated)
         self.db_mngr.alternatives_updated.connect(self.receive_alternatives_updated)
@@ -74,10 +98,14 @@ class SpineDBSignaller(QObject):
         self.db_mngr.parameter_definitions_updated.connect(self.receive_parameter_definitions_updated)
         self.db_mngr.parameter_values_updated.connect(self.receive_parameter_values_updated)
         self.db_mngr.parameter_value_lists_updated.connect(self.receive_parameter_value_lists_updated)
+        self.db_mngr.list_values_updated.connect(self.receive_list_values_updated)
         self.db_mngr.features_updated.connect(self.receive_features_updated)
         self.db_mngr.tools_updated.connect(self.receive_tools_updated)
         self.db_mngr.tool_features_updated.connect(self.receive_tool_features_updated)
         self.db_mngr.tool_feature_methods_updated.connect(self.receive_tool_feature_methods_updated)
+        self.db_mngr.metadata_updated.connect(self.receive_metadata_updated)
+        self.db_mngr.entity_metadata_updated.connect(self.receive_entity_metadata_updated)
+        self.db_mngr.parameter_value_metadata_updated.connect(self.receive_parameter_value_metadata_updated)
         # Removed
         self.db_mngr.scenarios_removed.connect(self.receive_scenarios_removed)
         self.db_mngr.alternatives_removed.connect(self.receive_alternatives_removed)
@@ -89,10 +117,14 @@ class SpineDBSignaller(QObject):
         self.db_mngr.parameter_definitions_removed.connect(self.receive_parameter_definitions_removed)
         self.db_mngr.parameter_values_removed.connect(self.receive_parameter_values_removed)
         self.db_mngr.parameter_value_lists_removed.connect(self.receive_parameter_value_lists_removed)
+        self.db_mngr.list_values_removed.connect(self.receive_list_values_removed)
         self.db_mngr.features_removed.connect(self.receive_features_removed)
         self.db_mngr.tools_removed.connect(self.receive_tools_removed)
         self.db_mngr.tool_features_removed.connect(self.receive_tool_features_removed)
         self.db_mngr.tool_feature_methods_removed.connect(self.receive_tool_feature_methods_removed)
+        self.db_mngr.metadata_removed.connect(self.receive_metadata_removed)
+        self.db_mngr.entity_metadata_removed.connect(self.receive_entity_metadata_removed)
+        self.db_mngr.parameter_value_metadata_removed.connect(self.receive_parameter_value_metadata_removed)
         # Commit, rollback, refresh
         self.db_mngr.session_refreshed.connect(self.receive_session_refreshed)
         self.db_mngr.session_committed.connect(self.receive_session_committed)
@@ -139,6 +171,10 @@ class SpineDBSignaller(QObject):
     def receive_parameter_value_lists_added(self, db_map_data):
         self._call_in_listeners("receive_parameter_value_lists_added", db_map_data)
 
+    @Slot(object)
+    def receive_list_values_added(self, db_map_data):
+        self._call_in_listeners("receive_list_values_added", db_map_data)
+
     @Slot(object)
     def receive_features_added(self, db_map_data):
         self._call_in_listeners("receive_features_added", db_map_data)
@@ -155,6 +191,18 @@ class SpineDBSignaller(QObject):
     def receive_tool_feature_methods_added(self, db_map_data):
         self._call_in_listeners("receive_tool_feature_methods_added", db_map_data)
 
+    @Slot(object)
+    def receive_metadata_added(self, db_map_data):
+        self._call_in_listeners("receive_metadata_added", db_map_data)
+
+    @Slot(object)
+    def receive_entity_metadata_added(self, db_map_data):
+        self._call_in_listeners("receive_entity_metadata_added", db_map_data)
+
+    @Slot(object)
+    def receive_parameter_value_metadata_added(self, db_map_data):
+        self._call_in_listeners("receive_parameter_value_metadata_added", db_map_data)
+
     @Slot(object)
     def receive_scenarios_updated(self, db_map_data):
         self._call_in_listeners("receive_scenarios_updated", db_map_data)
@@ -191,6 +239,10 @@ class SpineDBSignaller(QObject):
     def receive_parameter_value_lists_updated(self, db_map_data):
         self._call_in_listeners("receive_parameter_value_lists_updated", db_map_data)
 
+    @Slot(object)
+    def receive_list_values_updated(self, db_map_data):
+        self._call_in_listeners("receive_list_values_updated", db_map_data)
+
     @Slot(object)
     def receive_features_updated(self, db_map_data):
         self._call_in_listeners("receive_features_updated", db_map_data)
@@ -207,6 +259,18 @@ class SpineDBSignaller(QObject):
     def receive_tool_feature_methods_updated(self, db_map_data):
         self._call_in_listeners("receive_tool_feature_methods_updated", db_map_data)
 
+    @Slot(object)
+    def receive_metadata_updated(self, db_map_data):
+        self._call_in_listeners("receive_metadata_updated", db_map_data)
+
+    @Slot(object)
+    def receive_entity_metadata_updated(self, db_map_data):
+        self._call_in_listeners("receive_entity_metadata_updated", db_map_data)
+
+    @Slot(object)
+    def receive_parameter_value_metadata_updated(self, db_map_data):
+        self._call_in_listeners("receive_parameter_value_metadata_updated", db_map_data)
+
     @Slot(object)
     def receive_scenarios_removed(self, db_map_data):
         self._call_in_listeners("receive_scenarios_removed", db_map_data)
@@ -247,6 +311,10 @@ class SpineDBSignaller(QObject):
     def receive_parameter_value_lists_removed(self, db_map_data):
         self._call_in_listeners("receive_parameter_value_lists_removed", db_map_data)
 
+    @Slot(object)
+    def receive_list_values_removed(self, db_map_data):
+        self._call_in_listeners("receive_list_values_removed", db_map_data)
+
     @Slot(object)
     def receive_features_removed(self, db_map_data):
         self._call_in_listeners("receive_features_removed", db_map_data)
@@ -263,6 +331,18 @@ class SpineDBSignaller(QObject):
     def receive_tool_feature_methods_removed(self, db_map_data):
         self._call_in_listeners("receive_tool_feature_methods_removed", db_map_data)
 
+    @Slot(object)
+    def receive_metadata_removed(self, db_map_data):
+        self._call_in_listeners("receive_metadata_removed", db_map_data)
+
+    @Slot(object)
+    def receive_entity_metadata_removed(self, db_map_data):
+        self._call_in_listeners("receive_entity_metadata_removed", db_map_data)
+
+    @Slot(object)
+    def receive_parameter_value_metadata_removed(self, db_map_data):
+        self._call_in_listeners("receive_parameter_value_metadata_removed", db_map_data)
+
     @Slot(object)
     def receive_error_msg(self, db_map_error_log):
         self._call_in_listeners("receive_error_msg", db_map_error_log)
@@ -280,7 +360,10 @@ class SpineDBSignaller(QObject):
                     method = getattr(listener, callback)
                 except AttributeError:
                     continue
-                method(shared_db_map_data)
+                for db_map, data in shared_db_map_data.items():
+                    if self._defer_notification(db_map, data, method):
+                        continue
+                    method({db_map: data})
 
     @Slot(set)
     def receive_session_refreshed(self, db_maps):
