@@ -74,7 +74,19 @@ class SpineEngineManagerBase:
             command (str): command to issue
 
         Returns:
-            generator: stdio and stderr messages (dictionaries with two keys: type, and data)
+            generator: stdin, stdout, and stderr messages (dictionaries with two keys: type, and data)
+        """
+        raise NotImplementedError()
+
+    def is_persistent_command_complete(self, persistent_key, command):
+        """Checkes whether a command is complete.
+
+        Args:
+            key (tuple): persistent identifier
+            cmd (str): command to issue
+
+        Returns:
+            bool
         """
         raise NotImplementedError()
 
@@ -83,6 +95,9 @@ class SpineEngineManagerBase:
 
         Args:
             persistent_key (tuple): persistent identifier
+
+        Returns:
+            generator: stdout and stderr messages (dictionaries with two keys: type, and data)
         """
         raise NotImplementedError()
 
@@ -160,6 +175,10 @@ class RemoteSpineEngineManager(SpineEngineManagerBase):
         """See base class."""
         raise NotImplementedError()
 
+    def is_persistent_command_complete(self, persistent_key, command):
+        """See base class."""
+        raise NotImplementedError()
+
     def restart_persistent(self, persistent_key):
         """See base class."""
         raise NotImplementedError()
@@ -250,11 +269,17 @@ class LocalSpineEngineManager(SpineEngineManagerBase):
 
         yield from issue_persistent_command(persistent_key, command)
 
+    def is_persistent_command_complete(self, persistent_key, command):
+        # pylint: disable=import-outside-toplevel
+        from spine_engine.execution_managers.persistent_execution_manager import is_persistent_command_complete
+
+        return is_persistent_command_complete(persistent_key, command)
+
     def restart_persistent(self, persistent_key):
         # pylint: disable=import-outside-toplevel
         from spine_engine.execution_managers.persistent_execution_manager import restart_persistent
 
-        restart_persistent(persistent_key)
+        yield from restart_persistent(persistent_key)
 
     def interrupt_persistent(self, persistent_key):
         # pylint: disable=import-outside-toplevel
