@@ -29,8 +29,7 @@ from spinetoolbox.widgets.project_item_drag import ProjectItemDragMixin
 from spinetoolbox.config import JUPYTER_KERNEL_TIME_TO_DEAD
 from spinetoolbox.widgets.kernel_editor import find_kernels
 from spinetoolbox.spine_engine_manager import make_engine_manager
-from spinetoolbox.widgets.settings_widget import resolve_conda_executable
-from spine_engine.execution_managers.conda_kernel_spec_manager import CondaKernelSpecManager
+
 
 # Set logging level for jupyter loggers
 traitlets_logger = logging.getLogger("traitlets")
@@ -74,7 +73,7 @@ class JupyterConsoleWidget(RichJupyterWidget):
 
     @property
     def owner_names(self):
-        return "&".join(x.name for x in self.owners if x is not None)
+        return " & ".join(x.name for x in self.owners if x is not None)
 
     @Slot(bool)
     def start_console(self, checked=False):
